@@ -2,21 +2,18 @@
 
 #include <cassert>
 #include <cinttypes>
-#include <cstdlib>
 #include <cstring>
-#include <fstream>
 #include <iostream>
 #include <sqlite3.h>
 #include <sstream>
 #include <time.h>
 
-#include "config.h"
 #include "configcontainer.h"
-#include "controller.h"
 #include "dbexception.h"
 #include "logger.h"
 #include "matcherexception.h"
 #include "rssfeed.h"
+#include "rssignores.h"
 #include "scopemeasure.h"
 #include "strprintf.h"
 #include "utils.h"
@@ -263,8 +260,7 @@ static int guid_callback(void* myguids, int argc, char** argv,
 }
 
 Cache::Cache(const std::string& cachefile, ConfigContainer* c)
-	: db(0)
-	, cfg(c)
+	: cfg(c)
 {
 	const int error = sqlite3_open(cachefile.c_str(), &db);
 	if (error != SQLITE_OK) {
@@ -286,7 +282,7 @@ Cache::Cache(const std::string& cachefile, ConfigContainer* c)
 
 Cache::~Cache()
 {
-	sqlite3_close(db);
+	close_database();
 }
 
 void Cache::set_pragmas()
@@ -478,6 +474,10 @@ void Cache::update_lastmodified(const std::string& feedurl,
 			"empty, not updating anything");
 		return;
 	}
+
+	run_sql(prepare_query("INSERT OR IGNORE INTO rss_feed (rssurl, url, title) VALUES ('%q', '', '')",
+			feedurl));
+
 	std::lock_guard<std::recursive_mutex> lock(mtx);
 	std::string query = "UPDATE rss_feed SET ";
 	if (t > 0) {
@@ -780,8 +780,7 @@ void Cache::do_vacuum()
 std::vector<std::string> Cache::cleanup_cache(std::vector<std::shared_ptr<RssFeed>> feeds,
 	bool always_clean)
 {
-	// we don't use the std::lock_guard<> here... see comments below
-	mtx.lock();
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 
 	std::vector<std::string> unreachable_feeds{};
 	std::string list = "(";
@@ -846,9 +845,9 @@ std::vector<std::string> Cache::cleanup_cache(std::vector<std::shared_ptr<RssFee
 		run_sql(query, vectorofstring_callback, &unreachable_feeds);
 	}
 
-	// WARNING: THE MISSING UNLOCK OPERATION IS MISSING FOR A
-	// PURPOSE! It's missing so that no database operation can occur
-	// after the cache cleanup! mtx->unlock();
+	// Ensure that no other operations can occur after the cache cleanup
+	close_database();
+
 	return unreachable_feeds;
 }
 
@@ -1227,4 +1226,12 @@ SchemaVersion Cache::get_schema_version()
 	return result;
 }
 
+void Cache::close_database()
+{
+	if (db != nullptr) {
+		sqlite3_close(db);
+		db = nullptr;
+	}
+}
+
 } // namespace newsboat
