@@ -5,8 +5,6 @@
 #include <cstdint>
 #include <cstring>
 #include <langinfo.h>
-#include <numeric>
-#include <sstream>
 #include <string>
 
 #include "config.h"
@@ -275,7 +273,7 @@ REDO:
 			bool reload_only_visible_feeds =
 				cfg->get_configvalue_as_bool(
 					"reload-only-visible-feeds");
-			std::vector<int> idxs;
+			std::vector<unsigned int> idxs;
 			if (reload_only_visible_feeds) {
 				if (visible_feeds.empty()) {
 					// Do not reload *all* feeds, when
@@ -624,10 +622,10 @@ void FeedListFormAction::set_feedlist(
 	update_visible_feeds(feeds);
 
 	auto render_line = [this, feedlist_format](std::uint32_t line,
-	std::uint32_t width) -> std::string {
+	std::uint32_t width) -> StflRichText {
 		if (line >= visible_feeds.size())
 		{
-			return "ERROR";
+			return StflRichText::from_plaintext("ERROR");
 		}
 		auto& feed = visible_feeds[line];
 		return format_line(feedlist_format, feed.first, feed.second, width);
@@ -988,7 +986,7 @@ void FeedListFormAction::op_start_search()
 		"`%s'",
 		searchphrase);
 	if (searchphrase.length() > 0) {
-		const auto message_lifetime = v.get_statusline().show_message_until_finished(
+		auto message_lifetime = v.get_statusline().show_message_until_finished(
 				_("Searching..."));
 		searchhistory.add_line(searchphrase);
 		std::vector<std::shared_ptr<RssItem>> items;
@@ -1003,6 +1001,7 @@ void FeedListFormAction::op_start_search()
 					e.what()));
 			return;
 		}
+		message_lifetime.reset();
 		if (!items.empty()) {
 			std::shared_ptr<RssFeed> search_dummy_feed(new RssFeed(cache, ""));
 			search_dummy_feed->set_search_feed(true);
@@ -1061,7 +1060,7 @@ std::string FeedListFormAction::get_title(std::shared_ptr<RssFeed> feed)
 	return title;
 }
 
-std::string FeedListFormAction::format_line(const std::string& feedlist_format,
+StflRichText FeedListFormAction::format_line(const std::string& feedlist_format,
 	std::shared_ptr<RssFeed> feed,
 	unsigned int pos,
 	unsigned int width)
@@ -1084,19 +1083,20 @@ std::string FeedListFormAction::format_line(const std::string& feedlist_format,
 	fmt.register_fmt('L', utils::censor_url(feed->rssurl()));
 	fmt.register_fmt('d', utils::utf8_to_locale(feed->description()));
 
-	auto formattedLine = fmt.do_format(feedlist_format, width);
-	formattedLine = utils::quote_for_stfl(formattedLine);
+	const auto formattedLine = fmt.do_format(feedlist_format, width);
+	auto stflFormattedLine = StflRichText::from_plaintext(formattedLine);
+
+	if (unread_count > 0) {
+		stflFormattedLine.apply_style_tag("<unread>", 0, formattedLine.length());
+	}
 
 	const int id = rxman.feed_matches(feed.get());
 	if (id != -1) {
-		formattedLine = strprintf::fmt("<%d>%s</>", id, formattedLine);
+		const auto tag = strprintf::fmt("<%d>", id);
+		stflFormattedLine.apply_style_tag(tag, 0, formattedLine.length());
 	}
 
-	if (unread_count > 0) {
-		formattedLine = strprintf::fmt("<unread>%s</>", formattedLine);
-	}
-
-	return formattedLine;
+	return stflFormattedLine;
 }
 
 std::string FeedListFormAction::title()
