@@ -1,13 +1,10 @@
 #include "regexmanager.h"
 
-#include <cstring>
-#include <iostream>
-#include <stack>
-
 #include "config.h"
 #include "confighandlerexception.h"
 #include "configparser.h"
 #include "logger.h"
+#include "stflrichtext.h"
 #include "strprintf.h"
 #include "utils.h"
 
@@ -78,104 +75,12 @@ void RegexManager::remove_last_regex(const std::string& location)
 	regexes.pop_back();
 }
 
-std::map<size_t, std::string> RegexManager::extract_style_tags(std::string& str)
-{
-	std::map<size_t, std::string> tags;
-
-	size_t pos = 0;
-	while (pos < str.size()) {
-		auto tag_start = str.find_first_of("<>", pos);
-		if (tag_start == std::string::npos) {
-			break;
-		}
-		if (str[tag_start] == '>') {
-			// Keep unmatched '>' (stfl way of encoding a literal '>')
-			pos = tag_start + 1;
-			continue;
-		}
-		auto tag_end = str.find_first_of("<>", tag_start + 1);
-		if (tag_end == std::string::npos) {
-			break;
-		}
-		if (str[tag_end] == '<') {
-			// First '<' bracket is unmatched, ignoring it
-			pos = tag_start + 1;
-			continue;
-		}
-		if (tag_end - tag_start == 1) {
-			// Convert "<>" into "<" (stfl way of encoding a literal '<')
-			str.erase(tag_end, 1);
-			pos = tag_start + 1;
-			continue;
-		}
-		tags[tag_start] = str.substr(tag_start, tag_end - tag_start + 1);
-		str.erase(tag_start, tag_end - tag_start + 1);
-		pos = tag_start;
-	}
-	return tags;
-}
-
-void RegexManager::insert_style_tags(std::string& str,
-	std::map<size_t, std::string>& tags)
-{
-	// Expand "<" into "<>" (reverse of what happened in extract_style_tags()
-	size_t pos = 0;
-	while (pos < str.size()) {
-		auto bracket = str.find_first_of("<", pos);
-		if (bracket == std::string::npos) {
-			break;
-		}
-		pos = bracket + 1;
-		// Add to strings in the `tags` map so we don't have to shift all the positions in that map
-		// (would be necessary if inserting directly into `str`
-		tags[pos] = ">" + tags[pos];
-	}
-
-	for (auto it = tags.rbegin(); it != tags.rend(); ++it) {
-		if (it->first > str.length()) {
-			// Ignore tags outside of string
-			continue;
-		}
-		str.insert(it->first, it->second);
-	}
-}
-
-void RegexManager::merge_style_tag(std::map<size_t, std::string>& tags,
-	const std::string& tag, size_t start, size_t end)
-{
-	if (end <= start) {
-		return;
-	}
-
-	// Find the latest tag occurring before `end`.
-	// It is important that looping executes in ascending order of location.
-	std::string latest_tag = "</>";
-	for (const auto& location_tag : tags) {
-		size_t location = location_tag.first;
-		if (location > end) {
-			break;
-		}
-		latest_tag = location_tag.second;
-	}
-	tags[start] = tag;
-	tags[end] = latest_tag;
-
-	// Remove any old tags between the start and end marker
-	for (auto it = tags.begin(); it != tags.end(); ) {
-		if (it->first > start && it->first < end) {
-			it = tags.erase(it);
-		} else {
-			++it;
-		}
-	}
-}
-
-void RegexManager::quote_and_highlight(std::string& str,
+void RegexManager::quote_and_highlight(StflRichText& stflString,
 	const std::string& location)
 {
 	auto& regexes = locations[location];
 
-	auto tag_locations = extract_style_tags(str);
+	const std::string text = stflString.plaintext();
 
 	for (unsigned int i = 0; i < regexes.size(); ++i) {
 		const auto& regex = regexes[i].first;
@@ -184,8 +89,8 @@ void RegexManager::quote_and_highlight(std::string& str,
 		}
 		unsigned int offset = 0;
 		int eflags = 0;
-		while (offset < str.length()) {
-			const auto matches = regex->matches(str.substr(offset), 1, eflags);
+		while (offset < text.length()) {
+			const auto matches = regex->matches(text.substr(offset), 1, eflags);
 			eflags |= REG_NOTBOL; // Don't match beginning-of-line operator (^) in following checks
 			if (matches.empty()) {
 				break;
@@ -195,15 +100,13 @@ void RegexManager::quote_and_highlight(std::string& str,
 				const std::string marker = strprintf::fmt("<%u>", i);
 				const int match_start = offset + match.first;
 				const int match_end = offset + match.second;
-				merge_style_tag(tag_locations, marker, match_start, match_end);
+				stflString.apply_style_tag(marker, match_start, match_end);
 				offset = match_end;
 			} else {
 				offset++;
 			}
 		}
 	}
-
-	insert_style_tags(str, tag_locations);
 }
 
 void RegexManager::handle_highlight_action(const std::vector<std::string>&
