@@ -1,7 +1,6 @@
 #include "reloader.h"
 
 #include <algorithm>
-#include <cinttypes>
 #include <iostream>
 #include <ncurses.h>
 #include <thread>
@@ -11,6 +10,7 @@
 #include "dbexception.h"
 #include "feedretriever.h"
 #include "fmtstrformatter.h"
+#include "logger.h"
 #include "matcherexception.h"
 #include "reloadthread.h"
 #include "rss/exception.h"
@@ -35,7 +35,7 @@ void Reloader::spawn_reloadthread()
 	t.detach();
 }
 
-void Reloader::start_reload_all_thread(const std::vector<int>& indexes)
+void Reloader::start_reload_all_thread(const std::vector<unsigned int>& indexes)
 {
 	LOG(Level::INFO, "starting reload all thread");
 	std::thread t([=]() {
@@ -64,22 +64,17 @@ bool Reloader::trylock_reload_mutex()
 	return false;
 }
 
-void Reloader::reload(unsigned int pos,
-	bool show_progress,
-	bool unattended)
-{
-	CurlHandle handle;
-	reload(pos, handle, show_progress, unattended);
-}
-
 void Reloader::reload(unsigned int pos,
 	CurlHandle& easyhandle,
 	bool show_progress,
 	bool unattended)
 {
+	ScopeMeasure sm("Reloader::reload");
 	LOG(Level::DEBUG, "Reloader::reload: pos = %u", pos);
 	std::shared_ptr<RssFeed> oldfeed = ctrl->get_feedcontainer()->get_feed(pos);
 	if (oldfeed) {
+		LOG(Level::INFO, "Reloader::reload: starting reload of %s", oldfeed->rssurl());
+
 		// Query feed reloading should be handled by the calling functions
 		// (e.g.  Reloader::reload_all() calling View::prepare_query_feed())
 		if (oldfeed->is_query_feed()) {
@@ -110,13 +105,16 @@ void Reloader::reload(unsigned int pos,
 			RssIgnores* ign = ignore_dl ? ctrl->get_ignores() : nullptr;
 
 			LOG(Level::INFO, "Reloader::reload: retrieving feed");
-			FeedRetriever feed_retriever(cfg, *rsscache, ign, ctrl->get_api(), &easyhandle);
+			sm.stopover("start retrieving");
+			FeedRetriever feed_retriever(cfg, *rsscache, easyhandle, ign, ctrl->get_api());
 			const rsspp::Feed feed = feed_retriever.retrieve(oldfeed->rssurl());
 
 			LOG(Level::INFO, "Reloader::reload: parsing feed");
+			sm.stopover("start parsing");
 			RssParser parser(oldfeed->rssurl(), *rsscache, cfg, ign);
 
 			std::shared_ptr<RssFeed> newfeed = parser.parse(feed);
+			sm.stopover("start replacing feed");
 			if (newfeed != nullptr) {
 				ctrl->replace_feed(
 					oldfeed, newfeed, pos, unattended);
@@ -136,11 +134,14 @@ void Reloader::reload(unsigned int pos,
 					_("Error while retrieving %s: %s"),
 					utils::censor_url(oldfeed->rssurl()),
 					emsg);
-		} catch (rsspp::Exception& e) {
+		} catch (const rsspp::Exception& e) {
 			errmsg = strprintf::fmt(
 					_("Error while retrieving %s: %s"),
 					utils::censor_url(oldfeed->rssurl()),
 					e.what());
+		} catch (const rsspp::NotModifiedException&) {
+			// Nothing to be done, feed was not chaned since last retrieve
+			oldfeed->set_status(DlStatus::SUCCESS);
 		}
 		if (!errmsg.empty()) {
 			oldfeed->set_status(DlStatus::DL_ERROR);
@@ -203,9 +204,11 @@ void Reloader::reload_all(bool unattended)
 	ctrl->get_feedcontainer()->reset_feeds_status();
 	const auto num_feeds = ctrl->get_feedcontainer()->feeds_size();
 
-	partition_reload_to_threads([=](unsigned int start, unsigned int end) {
-		reload_range(start, end, unattended);
-	}, num_feeds);
+	std::vector<unsigned int> v;
+	for (unsigned int i = 0; i < num_feeds; ++i) {
+		v.push_back(i);
+	}
+	reload_indexes_impl(v, unattended);
 
 	// refresh query feeds (update and sort)
 	LOG(Level::DEBUG, "Reloader::reload_all: refresh query feeds");
@@ -227,32 +230,8 @@ void Reloader::reload_all(bool unattended)
 	notify_reload_finished(unread_feeds, unread_articles);
 }
 
-void Reloader::reload_indexes(const std::vector<int>& indexes, bool unattended)
+void Reloader::reload_indexes_impl(std::vector<unsigned int> indexes, bool unattended)
 {
-	ScopeMeasure m1("Reloader::reload_indexes");
-	const auto unread_feeds =
-		ctrl->get_feedcontainer()->unread_feed_count();
-	const auto unread_articles =
-		ctrl->get_feedcontainer()->unread_item_count();
-
-	partition_reload_to_threads([&](unsigned int start, unsigned int end) {
-		for (auto i = start; i <= end; ++i) {
-			reload(indexes[i], true, unattended);
-		}
-	}, indexes.size());
-
-	notify_reload_finished(unread_feeds, unread_articles);
-}
-
-void Reloader::reload_range(unsigned int start,
-	unsigned int end,
-	bool unattended)
-{
-	std::vector<unsigned int> v;
-	for (unsigned int i = start; i <= end; ++i) {
-		v.push_back(i);
-	}
-
 	auto extract = [](std::string& s, const std::string& url) {
 		size_t p = url.find("//");
 		p = (p == std::string::npos) ? 0 : p + 2;
@@ -263,7 +242,11 @@ void Reloader::reload_range(unsigned int start,
 
 	const auto feeds = ctrl->get_feedcontainer()->get_all_feeds();
 
-	std::sort(v.begin(), v.end(), [&](unsigned int a, unsigned int b) {
+	// Sort the feeds based on domain, so feeds on the same domain
+	// can share the curl handle.
+	//
+	// See commit: 115cf667485929bbb698ba8051dec7ff6a73739c
+	std::sort(indexes.begin(), indexes.end(), [&](unsigned int a, unsigned int b) {
 		std::string domain1, domain2;
 		extract(domain1, feeds[a]->rssurl());
 		extract(domain2, feeds[b]->rssurl());
@@ -272,14 +255,31 @@ void Reloader::reload_range(unsigned int start,
 		return domain1 < domain2;
 	});
 
-	CurlHandle easyhandle;
+	partition_reload_to_threads([&](unsigned int start, unsigned int end) {
+		CurlHandle easyhandle;
+		for (auto i = start; i <= end; ++i) {
+			unsigned int feed_index = indexes[i];
+			LOG(Level::DEBUG,
+				"Reloader::reload_indexes_impl: reloading feed #%u",
+				feed_index);
+			reload(feed_index, easyhandle, true, unattended);
+			// Reset any options set on the handle before next reload
+			curl_easy_reset(easyhandle.ptr());
+		}
+	}, indexes.size());
+}
 
-	for (const auto& i : v) {
-		LOG(Level::DEBUG,
-			"Reloader::reload_range: reloading feed #%u",
-			i);
-		reload(i, easyhandle, true, unattended);
-	}
+void Reloader::reload_indexes(const std::vector<unsigned int>& indexes, bool unattended)
+{
+	ScopeMeasure m1("Reloader::reload_indexes");
+	const auto unread_feeds =
+		ctrl->get_feedcontainer()->unread_feed_count();
+	const auto unread_articles =
+		ctrl->get_feedcontainer()->unread_item_count();
+
+	reload_indexes_impl(indexes, unattended);
+
+	notify_reload_finished(unread_feeds, unread_articles);
 }
 
 void Reloader::notify(const std::string& msg)
