@@ -1,31 +1,23 @@
 #include "utils.h"
-#include <iostream>
 
-#include <algorithm>
 #include <cinttypes>
-#include <cmath>
-#include <cstdarg>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <curl/curl.h>
 #include <cwchar>
-#include <errno.h>
 #include <fcntl.h>
 #include <iconv.h>
 #include <langinfo.h>
 #include <libxml/uri.h>
-#include <locale>
 #include <mutex>
 #include <pwd.h>
-#include <regex>
 #include <sstream>
 #include <stfl.h>
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/utsname.h>
 #include <unistd.h>
-#include <unordered_set>
 
 #if HAVE_GCRYPT
 #include <gnutls/gnutls.h>
@@ -47,7 +39,8 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #include "config.h"
 #include "curldatareceiver.h"
 #include "curlhandle.h"
-#include "htmlrenderer.h"
+#include "links.h"
+#include "libnewsboat-ffi/src/utils.rs.h"
 #include "logger.h"
 #include "strprintf.h"
 
@@ -192,6 +185,19 @@ std::string utils::locale_to_utf8(const std::string& text)
 	return std::string(utils::bridged::locale_to_utf8(text_slice));
 }
 
+std::string utils::convert_text(const std::string& text, const std::string& tocode,
+	const std::string& fromcode)
+{
+	const auto text_slice =
+		rust::Slice<const unsigned char>(
+			reinterpret_cast<const unsigned char*>(text.c_str()),
+			text.length());
+
+	const auto result = utils::bridged::convert_text(text_slice, tocode, fromcode);
+
+	return std::string(reinterpret_cast<const char*>(result.data()), result.size());
+}
+
 std::string utils::get_command_output(const std::string& cmd)
 {
 	return std::string(utils::bridged::get_command_output(cmd));
@@ -383,6 +389,17 @@ std::string utils::replace_all(const std::string& str,
 	return output;
 }
 
+std::string utils::to_lowercase(const std::string& input)
+{
+	std::string output;
+	std::transform(input.begin(), input.end(), std::back_inserter(output),
+	[](unsigned char c) {
+		return std::tolower(c);
+	});
+
+	return output;
+}
+
 std::string utils::preserve_quotes(const std::string& str)
 {
 	std::string escaped_string = "";
@@ -773,6 +790,13 @@ nonstd::optional<LinkType> utils::podcast_mime_to_link_type(
 	return nonstd::nullopt;
 }
 
+std::string utils::string_from_utf8_lossy(const std::vector<std::uint8_t>& text)
+{
+	auto input = rust::Slice<std::uint8_t const>(text.data(), text.size());
+	auto result = utils::bridged::string_from_utf8_lossy(input);
+	return std::string(result);
+}
+
 /*
  * See
  * http://curl.haxx.se/libcurl/c/libcurl-tutorial.html#Multi-threading
